/*
 * myfile
 *
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.tizenopensource.org/license
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */




#include <pthread.h>
#include <vconf.h>
#include <haptic.h>

#include "mf-ug-util.h"
#include "mf-ug-cb.h"
#include "mf-ug-winset.h"
#include "mf-ug-fm-svc-wrapper.h"
#include "mf-ug-fs-util.h"
#include "mf-ug-inotify-handle.h"
#include "mf-ug-resource.h"

#define MF_UG_TIMER_INTERVAL_VIBRATION 0.5
#define MF_UG_VIBRATION_DEVICE 0
#define MF_UG_VIBRATION_DURATION 500

/******************************
** Prototype    : ug_genlist_selected_state_get
** Description  :
** Input        : void *data
** Output       : bool
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
bool mf_ug_util_is_genlist_selected(void *data)
{
	UG_TRACE_BEGIN;
	ugData *ugd = (ugData *)data;
	ug_mf_retvm_if(ugd == NULL, false, "ugd is NULL");

	Elm_Object_Item *gli = NULL;
	Elm_Object_Item *nli = NULL;
	Evas_Object *genlist = NULL;

	ugNaviBar *pNavi_s = mf_ug_navi_bar_get_in_use_navi(ugd);
	ug_mf_retvm_if(pNavi_s == NULL, false, "get navigation bar in use failed");

	genlist = ugd->ug_MainWindow.ug_pContent;
	gli = elm_genlist_first_item_get(genlist);

	while (gli) {
		ugListItemData *itemData = (ugListItemData *)elm_object_item_data_get(gli);
		ug_mf_retvm_if(itemData == NULL, false, "itemData is NULL");
		if (itemData->ug_bChecked == true) {
			UG_TRACE_END;
			return true;
		}
		nli = elm_genlist_item_next_get(gli);
		gli = nli;
	}
	UG_TRACE_END;
	return false;
}

/******************************
** Prototype    : ug_parase_path_get
** Description  :
** Input        : GList *dest_list
**                char *path_list
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_ug_util_get_params_path(Eina_List **dest_list, const char *path_list)
{
	UG_TRACE_BEGIN;
	ug_mf_retm_if(dest_list == NULL, "dest_list is NULL");

	gchar **result = NULL;
	gchar **params = NULL;
	result = g_strsplit(path_list, "?", 0);

	for (params = result; *params; params++) {
		*dest_list = eina_list_append(*dest_list, strdup(*params));
	}

	g_strfreev(result);
	UG_TRACE_END;
}

/******************************
** Prototype    : mf_ug_util_free_eina_list_data
** Description  : Samsung
** Input        : Eina_List **list
**                mf_ug_eina_list_node_type node_type
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_ug_util_free_eina_list_data(Eina_List **list, mf_ug_eina_list_node_type node_type)
{
	UG_TRACE_BEGIN;
	ug_mf_retm_if(list == NULL, "list is NULL");
	ug_mf_retm_if(*list == NULL, "*list is NULL");

	Eina_List *l = NULL;
	void *pNode = NULL;
	switch (node_type) {
	case NODE_TYPE_CHAR:
	case NODE_TYPE_PNODE:
		EINA_LIST_FOREACH(*list, l, pNode) {
			free(pNode);
			pNode = NULL;
		}
		break;
	case NODE_TYPE_GSTRING:
		EINA_LIST_FOREACH(*list, l, pNode) {
			if (pNode != NULL) {
				g_string_free(pNode, TRUE);
				pNode = NULL;
			}
		}
		break;
	default:
		break;
	}

	eina_list_free(*list);
	*list = NULL;
	UG_TRACE_END;
}

/******************************
** Prototype    : mf_ug_util_is_mass_storage_on
** Description  : Samsung
** Input        : None
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
bool mf_ug_util_is_mass_storage_on()
{
	UG_TRACE_BEGIN;
	int usb_state = 0;
	int error_code = vconf_get_int(VCONFKEY_USB_STORAGE_STATUS, &usb_state);
	ug_debug("usb state is %d", usb_state);
	ug_debug("error_code is %d", error_code);
	if (error_code == 0) {
		if (usb_state != VCONFKEY_USB_STORAGE_STATUS_UMS_ON && usb_state != VCONFKEY_USB_STORAGE_STATUS_UMS_MMC_ON) {
			UG_TRACE_END;
			return false;
		} else {
			UG_TRACE_END;
			return true;
		}
	} else {
		UG_TRACE_END;
		return false;
	}
}


/******************************
** Prototype    : __mf_ug_util_get_marked_selected_items
** Description  :
** Input        : GList *list
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
static void __mf_ug_util_get_marked_selected_items(Eina_List *list, Eina_List **select_list)
{
	UG_TRACE_BEGIN;
	ug_mf_retm_if(select_list == NULL, "select_list is NULL");
	ug_mf_retm_if(list == NULL, "list is NULL");

	Eina_List *l = NULL;
	void *pNode = NULL;

	EINA_LIST_FOREACH(list, l, pNode) {
		if (pNode != NULL) {
			*select_list = eina_list_append(*select_list, strdup(pNode));
		}
	}
	UG_TRACE_END;

}

/******************************
** Prototype    : __mf_ug_util_get_marked_off_selected_items
** Description  :
** Input        : void *data
**                Evas_Object* content
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
static void __mf_ug_util_get_marked_off_selected_items(void *data, Eina_List **select_list)
{
	UG_TRACE_BEGIN;
	ugData *ugd = (ugData *)data;
	ug_mf_retm_if(ugd == NULL, "ugData is NULL");
	ug_mf_retm_if(select_list == NULL, "selected_list is NULL");


	Evas_Object *content = ugd->ug_MainWindow.ug_pContent;
	Elm_Object_Item *gli = elm_genlist_first_item_get(content);
	int radio_id = 1;
	Elm_Object_Item *nli = NULL;


	while (gli) {

		ugListItemData *params = (ugListItemData *)elm_object_item_data_get(gli);
		ug_mf_retm_if(params == NULL, "params is NULL");
		if (ugd->ug_UiGadget.ug_iSelectMode == SINGLE_ALL_MODE || ugd->ug_UiGadget.ug_iSelectMode == SINGLE_FILE_MODE) {
			if (params->ug_pRadioBox) {
				if (elm_radio_value_get(ugd->ug_MainWindow.ug_pRadioGroup) == radio_id) {
					*select_list = eina_list_append(*select_list, strdup(params->ug_pItemName->str));
					break;
				}
				radio_id++;
			}
		} else if (ugd->ug_UiGadget.ug_iSelectMode == MULTI_ALL_MODE ||
			   ugd->ug_UiGadget.ug_iSelectMode == MULTI_FILE_MODE ||
			   ugd->ug_UiGadget.ug_iSelectMode == IMPORT_MODE) {
			if (params->ug_pCheckBox) {
				if (params->ug_bChecked == true) {
					*select_list = eina_list_append(*select_list, strdup(params->ug_pItemName->str));
				}
			}
		}
		nli = elm_genlist_item_next_get(gli);
		gli = nli;
	}
	UG_TRACE_END;
}

/******************************
** Prototype    : mf_ug_util_get_send_result
** Description  :
** Input        : void *data
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
char *mf_ug_util_get_send_result(void *data)
{
	UG_TRACE_BEGIN;
	ugData *ugd = (ugData *)data;
	ug_mf_retvm_if(ugd == NULL, NULL, "ugData is NULL");

	char *file_selected = NULL;
	Eina_List *selected_list = NULL;
	Eina_List *l = NULL;
	char *pNode = NULL;

	if (ugd->ug_UiGadget.ug_iMarkedMode == MARKED_ON
	    && (ugd->ug_UiGadget.ug_iSelectMode == MULTI_FILE_MODE || ugd->ug_UiGadget.ug_iSelectMode == MULTI_ALL_MODE)) {
		__mf_ug_util_get_marked_selected_items(ugd->ug_UiGadget.ug_pMultiSelectFileList, &selected_list);
	} else {
		__mf_ug_util_get_marked_off_selected_items(ugd, &selected_list);
	}

	EINA_LIST_FOREACH(selected_list, l, pNode) {
		if (pNode != NULL) {
			if (file_selected == NULL) {
				file_selected = g_strconcat(pNode, NULL);
			} else {
				char *temp = file_selected;
				file_selected = g_strconcat(file_selected, "?", pNode, NULL);
				free(temp);
			}
		}
	}
	mf_ug_util_free_eina_list_data(&selected_list, NODE_TYPE_CHAR);
	UG_TRACE_END;
	return file_selected;
}


/******************************
** Prototype    : _ug_mf_set_state_as
** Description  :
** Input        : struct ugmyfiledata* data
**                int state
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_ug_util_set_current_state(void *data, int state)
{
	UG_TRACE_BEGIN;
	ugData *ugd = (ugData *)data;
	ug_mf_retm_if(ugd == NULL, "ugd is NULL");

	ugd->ug_Status.ug_iState = state;
	UG_TRACE_END;
}


/******************************
** Prototype    : mf_ug_util_get_mmc_state
** Description  :
** Input        : int* mmc_card
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
int mf_ug_util_get_mmc_state(int *mmc_card)
{
	UG_TRACE_BEGIN;
	int data = UG_ERROR_RETURN;
	int error_code = MYFILE_ERR_NONE;

	ug_mf_retvm_if(mmc_card == NULL, MYFILE_ERR_SRC_ARG_INVALID, "mmc_card is NULL");

	*mmc_card = 0;
	error_code = vconf_get_int(VCONFKEY_SYSMAN_MMC_STATUS, &data);

	if (error_code) {
		UG_TRACE_END;
		return MYFILE_ERR_GET_VCONF_FAIL;
	}

	if (data != VCONFKEY_SYSMAN_MMC_MOUNTED) {
		*mmc_card = MMC_OFF;
	} else {
		*mmc_card = MMC_ON;
	}
	UG_TRACE_END;
	return error_code;
}

/******************************
** Prototype    : mf_ug_util_create_dir_monitor
** Description  : Samsung
** Input        : void *data
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
int mf_ug_util_create_dir_monitor(void *data)
{
	UG_TRACE_BEGIN;
	ugData *ugd = (ugData *)data;
	ug_mf_retvm_if(ugd == NULL, UG_ERROR_RETURN, "ugData is NULL");

	if (ugd->ug_UiGadget.ug_pInotifyPipe) {
		ecore_pipe_del(ugd->ug_UiGadget.ug_pInotifyPipe);
		ugd->ug_UiGadget.ug_pInotifyPipe = NULL;
	}
	ugd->ug_UiGadget.ug_pInotifyPipe = ecore_pipe_add(mf_ug_cb_dir_pipe_cb, (const void *)ugd);
	UG_TRACE_END;

	return mf_ug_inotify_handle_init_inotify();
}


/******************************
** Prototype    : mf_ug_util_add_dir_watch
** Description  : Samsung
** Input        : const char *path
**                void *data
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
int mf_ug_util_add_dir_watch(const char *path, void *data)
{
	return mf_ug_inotify_handle_add_inotify_watch(path, mf_ug_cb_dir_update_cb, data);
}

int mf_ug_util_remove_dir_watch(void)
{
	return mf_ug_inotify_handle_rm_inotify_watch();
}

/******************************
** Prototype    : mf_ug_util_set_mmc_state_cb
** Description  : Samsung
** Input        : void *data
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
int mf_ug_util_set_mmc_state_cb(void *data)
{
	UG_TRACE_BEGIN;
	ugData *ugd = (ugData *)data;
	ug_mf_retvm_if(ugd == NULL, UG_ERROR_RETURN, "ugd is NULL");

	int mmc_state = MMC_OFF;
	mf_ug_util_get_mmc_state(&mmc_state);
	ugd->ug_Status.ug_iMmcFlag = mmc_state;

	UG_TRACE_END;
	return vconf_notify_key_changed(VCONFKEY_SYSMAN_MMC_STATUS, mf_ug_cb_mmc_changed_cb, ugd);
}

/******************************
** Prototype    : mf_ug_util_destory_mmc_state_cb
** Description  : Samsung
** Input        : None
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_ug_util_destory_mmc_state_cb()
{
	UG_TRACE_BEGIN;
	vconf_ignore_key_changed(VCONFKEY_SYSMAN_MMC_STATUS, mf_ug_cb_mmc_changed_cb);
	UG_TRACE_END;
}

void mf_ug_util_destory_mass_storage_callback()
{
	UG_TRACE_BEGIN;
	vconf_ignore_key_changed(VCONFKEY_USB_STORAGE_STATUS, mf_ug_cb_mass_storage_changed_cb);
	UG_TRACE_END;
}

/******************************
** Prototype    : mf_ug_util_destory_dir_monitor
** Description  : Samsung
** Input        : void *data
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_ug_util_destory_dir_monitor(void *data)
{
	UG_TRACE_BEGIN;
	ugData *ugd = (ugData *)data;
	ug_mf_retm_if(ugd == NULL, "ugd is NULL");

	if (ugd->ug_UiGadget.ug_pInotifyPipe) {
		ecore_pipe_del(ugd->ug_UiGadget.ug_pInotifyPipe);
		ugd->ug_UiGadget.ug_pInotifyPipe = NULL;
	}

	mf_ug_inotify_handle_finalize_inotify();
	UG_TRACE_END;

	return;
}


/******************************
** Prototype    : mf_ug_util_storage_insert_action
** Description  : Samsung
** Input        : void *data
**                char* pItemLabel
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_ug_util_storage_insert_action(void *data, char *pItemLabel)
{
	UG_TRACE_BEGIN;
	ugData *ugd = (ugData *)data;
	ug_mf_retm_if(ugd == NULL, "ugd is NULL");

	Elm_Object_Item *pItemPhone = NULL;
	ugNaviBar *pNaviBarStruct = NULL;

	mf_ug_tab_bar_init(ugd);
	if (!mf_ug_fm_svc_wapper_is_root_path(ugd)) {
		return;
	}

	pNaviBarStruct = ug_mf_navi_bar_get_navi_from_navilist(ugd->ug_MainWindow.ug_pNaviBarList, MF_UG_LABEL_PHONE);
	elm_object_part_content_unset(ugd->ug_MainWindow.ug_pMainLayout, "elm.swallow.content");
	pItemPhone = mf_ug_tab_bar_get_item_by_label(ugd->ug_MainWindow.ug_pTabBar, pNaviBarStruct->ug_pNaviLabel);
	mf_ug_tab_bar_set_selected_item(pItemPhone);
	elm_object_part_content_set(ugd->ug_MainWindow.ug_pTabLayout, "elm.swallow.content", ugd->ug_MainWindow.ug_pNaviBar);

	elm_object_part_content_set(ugd->ug_MainWindow.ug_pTabLayout, "elm.swallow.tabbar", ugd->ug_MainWindow.ug_pTabBar);
	elm_object_part_content_set(ugd->ug_MainWindow.ug_pMainLayout, "elm.swallow.content", ugd->ug_MainWindow.ug_pTabLayout);

	if (ugd->ug_Status.ug_bMassStorageFlag) {
		mf_ug_tab_bar_set_item_disable(ugd->ug_MainWindow.ug_pTabBar, EINA_TRUE);
	}
	UG_TRACE_END;
}


/******************************
** Prototype    : mf_ug_util_set_main_layout_by_tab_item_count
** Description  : __mf_ug_main_free_evas_object
** Input        : void *data
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
int mf_ug_util_set_main_layout_by_tab_item_count(void *data)
{
	UG_TRACE_BEGIN;
	ugData *ugd = (ugData *)data;
	ug_mf_retvm_if(ugd == NULL, 0, "ugd is NULL");

	Evas_Object *pOldContent = NULL;
	pOldContent = ugd->ug_MainWindow.ug_pTabBar;
	if (eina_list_count(ugd->ug_MainWindow.ug_pNaviBarList) < 2) {
		/*no:   tab bar delete operation */
		elm_object_part_content_unset(ugd->ug_MainWindow.ug_pTabLayout, "elm.swallow.tabbar");
		evas_object_del(pOldContent);
		elm_object_part_content_unset(ugd->ug_MainWindow.ug_pTabLayout, "elm.swallow.content");

		elm_object_part_content_unset(ugd->ug_MainWindow.ug_pMainLayout, "elm.swallow.content");
		UG_SAFE_FREE_OBJ(ugd->ug_MainWindow.ug_pTabLayout);
		ugd->ug_MainWindow.ug_pTabBar = NULL;
		UG_TRACE_END;
		return 0;
	} else {
		/*yes:  do nothing */
		//elm_toolbar_item_selected_set(elm_toolbar_first_item_get(ugd->ug_MainWindow.ug_pTabBar), EINA_TRUE);
		UG_TRACE_END;
		return 1;
	}
}

/******************************
** Prototype    : mf_ug_util_get_file_launch_type
** Description  :
** Input        : char * path
** Output       : UG_MYFILE_LAUNCH_TYPE
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
mf_ug_launch_type mf_ug_util_get_file_launch_type(char *path)
{
	if (path == NULL) {
		UG_TRACE_END;
		return LAUNCH_TYPE_UNSUPPORT;
	}
	if (ecore_file_file_get(path) == NULL) {
		UG_TRACE_END;
		return LAUNCH_TYPE_UNSUPPORT;
	}
	if (ecore_file_is_dir(path) == 1) {
		UG_TRACE_END;
		return LAUNCH_TYPE_UNSUPPORT;
	}
	if (mf_ug_file_attr_is_drm_file(path) == 0) {
		mf_ug_drm_file_mime_type drm_file_type = mf_ug_fs_oper_get_drm_type(path);

		if (drm_file_type == MF_UG_DRM_RINGTONE_FILE) {
			UG_TRACE_END;
			return LAUNCH_TYPE_MUSIC;
		} else if (drm_file_type == MF_UG_DRM_IMAGE_FILE) {
			UG_TRACE_END;
			return LAUNCH_TYPE_IMAGE;
		} else if (drm_file_type == MF_UG_DRM_VIDEO_FILE) {
			UG_TRACE_END;
			return LAUNCH_TYPE_VIDEO;
		} else {
			UG_TRACE_END;
			return LAUNCH_TYPE_UNSUPPORT;
		}
	} else {
		mf_ug_fs_file_type category_t = 0;
		mf_ug_file_attr_get_file_category(path, &category_t);
		if (category_t == UG_FILE_TYPE_MUSIC || category_t == UG_FILE_TYPE_SOUND || category_t == UG_FILE_TYPE_VOICE) {
			UG_TRACE_END;
			return LAUNCH_TYPE_MUSIC;
		} else if (category_t == UG_FILE_TYPE_IMAGE) {
			UG_TRACE_END;
			return LAUNCH_TYPE_IMAGE;
		} else {
			UG_TRACE_END;
			return LAUNCH_TYPE_UNSUPPORT;
		}
	}
	UG_TRACE_END;
}

mf_ug_theme_type mf_ug_util_get_theme(void)
{
	mf_ug_theme_type theme = UG_THEME_INVALID;
	char *name;

	char *theme_name = vconf_get_str(VCONFKEY_SETAPPL_WIDGET_THEME_STR);
	ug_debug("theme is [%s]", theme_name);
	ug_mf_retvm_if(theme_name == NULL, UG_THEME_ERROR, "Failed to get vconf value");	/* file system exceptional handle */

	name = strrchr(theme_name, '/');

	if (!name)
		name = theme_name;
	else
		++name;

	ug_debug("name is [%s]", name);
	//SETTING_TRACE("name:%s", name);
	/* special theme process.. */
	if (0 == g_strcmp0("tizen-black.edj", name) || 0 == g_strcmp0("tizen-black-hd.edj", name)) {
		theme = UG_THEME_NBEAT_BLACK;
	} else if (0 == g_strcmp0("tizen.edj", name) || 0 == g_strcmp0("tizen-hd.edj", name)
	           || 0 == g_strcmp0("blue-hd.edj", name)) {
		theme = UG_THEME_NBEAT;
	} else {
		theme = UG_THEME_ERROR;
	}
	free(theme_name);
	theme_name = NULL;
	ug_debug("theme is [%d]", theme);
	return theme;
}


char *mf_ug_util_upper_folder_name_get(void *data, GString *fullpath)
{
	ug_mf_retvm_if(data == NULL, NULL, "data is NULL");
	ug_mf_retvm_if(fullpath == NULL, NULL, "fullpath is NULL");
	ug_mf_retvm_if(fullpath->str == NULL, NULL, "fullpath is NULL");

	GString *parent_path = NULL;
	GString *gName = NULL;
	char *upper_name = NULL;

	parent_path = mf_ug_fm_svc_wrapper_get_file_parent_path(fullpath);

	ug_mf_retvm_if(parent_path == NULL, NULL, "fullpath is NULL");
	ug_mf_retvm_if(parent_path->str == NULL, NULL, "fullpath is NULL");


	if (!g_strcmp0(parent_path->str, PHONE_FOLDER)) {
		upper_name = g_strdup(MF_UG_LABEL_PHONE);
	} else if (!g_strcmp0(parent_path->str, MEMORY_FOLDER)) {
		upper_name = g_strdup(MF_UG_LABEL_MMC);
	} else {
		gName = mf_ug_fm_svc_wapper_get_file_name(parent_path);
		UG_GSTRING_CHECK_NULL_GOTO(gName, FAILED_EXIT);
		upper_name = g_strdup(gName->str);
	}

	UG_CHAR_CHECK_NULL_GOTO(upper_name, FAILED_EXIT);
	UG_SAFE_FREE_GSTRING(parent_path);
	UG_SAFE_FREE_GSTRING(gName);
	return upper_name;

FAILED_EXIT:
	UG_SAFE_FREE_GSTRING(parent_path);
	UG_SAFE_FREE_GSTRING(gName);
	return NULL;

}

void mf_ug_util_sort_the_file_list(void *data)
{

	UG_TRACE_BEGIN;
	ug_mf_retm_if(data == NULL, "data is null");
	ugData *ugd = (ugData *)data;

	int iSortTypeValue = 0;

	iSortTypeValue = ugd->ug_Status.ug_iSortType;

	mf_ug_fs_oper_sort_list(&ugd->ug_UiGadget.ug_pFilterList, iSortTypeValue);
	/*need to sort folder items only By Name and Date*/
	mf_ug_fs_oper_sort_list(&ugd->ug_UiGadget.ug_pDirList, iSortTypeValue);
}

int mf_ug_util_check_disk_space(void *data)
{
	UG_TRACE_BEGIN;
	ug_mf_retvm_if(data == NULL, MYFILE_ERR_SRC_ARG_INVALID, "data is null");
	ugData *ugd = (ugData *)data;

	mf_ug_storage_type state = MF_UG_NONE;
	unsigned long free_space = 0;

	if (ugd->ug_Status.ug_iMore == UG_MORE_CREATE_FOLDER) {
		state = mf_ug_fm_svc_wapper_get_location(ugd->ug_Status.ug_pPath->str);
		free_space = mf_ug_fm_svc_wrapper_get_free_space(state);
		ug_debug("free size is %lu\n", free_space);
		/*
		 **     in vfat fs type, sector size is 16K.
		 **     it is to say that the limited size of the free space should be 16K
		 **     or it will report space used up.
		 **     check free_space == 0 can make sure at least 16K is free on the disk
		 **     while every dir takes 4K
		 */
		if (free_space == 0) {
			mf_ug_popup_create(ugd, UG_POPMODE_TEXT, NULL,
					   MF_UG_LABEL_NOT_ENOUGH_SPACE, MF_UG_LABEL_OK,
					   NULL, NULL, mf_ug_cb_warning_popup_cb, ugd);
			return MYFILE_ERR_NO_FREE_SPACE;
		}
	}

	return MYFILE_ERR_NONE;
}

static void __mf_ug_util_stop_vibration(void *data)
{
	UG_TRACE_BEGIN;
	ug_mf_retm_if(data == NULL, "data is null");
	ugData *ugd = (ugData *)data;

	int ret = -1;

	if (ugd->ug_Status.ug_haptic_handle >= 0) {
		ret = haptic_stop_device(MF_UG_VIBRATION_DEVICE);
		if (ret != 0)
			ug_debug("haptic_stop_device failed[%d]!", ret);

		ret = haptic_deinitialize();
		if (ret != HAPTIC_ERROR_NONE)
			ug_debug("haptic_deinitialize failed[%d]!", ret);
		ugd->ug_Status.ug_haptic_handle = -1;
	}
	UG_TRACE_END;
}

static Eina_Bool __mf_ug_util_vibration_timer_cb(void *data)
{
	UG_TRACE_BEGIN;
	ug_mf_retvm_if(data == NULL, FALSE, "data is null");
	ugData *ugd = (ugData *)data;

	ugd->ug_Status.vibration_timer = NULL;
	__mf_ug_util_stop_vibration(ugd);

	return ECORE_CALLBACK_DONE;
}


bool mf_ug_util_play_vibration(void *data, float duration)
{
	UG_TRACE_BEGIN;
	ug_mf_retvm_if(data == NULL, FALSE, "data is null");
	ug_mf_retvm_if(duration <= 0, FALSE, "Invalid parameter");
	ugData *ugd = (ugData *)data;
	int ret = -1;

	ret = haptic_initialize();
	if (ret != HAPTIC_ERROR_NONE) {
		ug_debug("haptic_initialize failed[%d]!", ret);
		return -1;
	}

	UG_SAFE_DEL_ECORE_TIMER(ugd->ug_Status.vibration_timer);
	ugd->ug_Status.vibration_timer = ecore_timer_add(MF_UG_TIMER_INTERVAL_VIBRATION, __mf_ug_util_vibration_timer_cb, ugd);

	ret = haptic_vibrate_monotone(MF_UG_VIBRATION_DEVICE, MF_UG_VIBRATION_DURATION, HAPTIC_LEVEL_3);


	if (ret != HAPTIC_ERROR_NONE) {
		ug_debug("haptic_play_pattern failed[%d]!", ret);
		return -1;
	}
	ugd->ug_Status.ug_haptic_handle = MF_UG_VIBRATION_DEVICE;

	return TRUE;
}

void mf_ug_util_operation_alloc_failed(void *data)
{
	UG_TRACE_BEGIN;
	ug_mf_retm_if(data == NULL, "data is null");
	ugData *ugd = (ugData *)data;

	mf_ug_popup_create(ugd, UG_POPMODE_TEXT, NULL, MF_UG_LABEL_MEMORY_NOT_ENOUGH,
			   NULL, NULL, NULL, (Evas_Smart_Cb) elm_exit, NULL);
}

