/*
 * myfile
 *
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.tizenopensource.org/license
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */



#include "mf-ug-winset.h"
#include "mf-ug-cb.h"
#include "mf-ug-fm-svc-wrapper.h"
#include "mf-ug-util.h"
#include "mf-ug-fs-util.h"
#include "mf-ug-resource.h"


/******************************
** Prototype    : __mf_ug_navi_bar_create_group_radio_box
** Description  : Samsung
** Input        : void *data
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
static void __mf_ug_navi_bar_create_group_radio_box(void *data)
{
	UG_TRACE_BEGIN;
	ugData *ugd = (ugData *)data;
	ug_mf_retm_if(ugd == NULL, "ugd is NULL");

	Evas_Object *group_radio = NULL;

	if (ugd->ug_MainWindow.ug_pRadioGroup != NULL) {
		evas_object_del(ugd->ug_MainWindow.ug_pRadioGroup);
		ugd->ug_MainWindow.ug_pRadioGroup = NULL;
	}

	group_radio = elm_radio_add(ugd->ug_MainWindow.ug_pMainLayout);
	ug_mf_retm_if(group_radio == NULL, "ugd is NULL");

	elm_radio_value_set(group_radio, 0);
	evas_object_hide(group_radio);
	/*/Internal/invisible Radio Objects used to group the radio buttons in the list. */
	ugd->ug_MainWindow.ug_pRadioGroup = group_radio;
	UG_TRACE_END;
}


/******************************
** Prototype    : __mf_ug_navi_bar_push_content
** Description  : Samsung
** Input        : ugData *data
**                Evas_Object *NaviContent
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
static void __mf_ug_navi_bar_push_content(void *data, Evas_Object *NaviContent)
{
	UG_TRACE_BEGIN;
	ug_mf_retm_if(data == NULL, "pNavi_s is NULL");
	ugData *ugd = (ugData *)data;
	ugNaviBar *pNavi_s = mf_ug_navi_bar_get_in_use_navi(ugd);

	Evas_Object *NaviBar = ugd->ug_MainWindow.ug_pNaviBar;
	char *title = pNavi_s->ug_pNaviTitle;

	ugd->ug_MainWindow.ug_pPreNaviItem = ugd->ug_MainWindow.ug_pNaviItem;

	ugd->ug_MainWindow.ug_pNaviItem = elm_naviframe_item_push(NaviBar, title, NULL, NULL, NaviContent, MF_UG_NAVI_STYLE_ENABLE);

	UG_TRACE_END;
}

static Evas_Object *__mf_ug_navi_bar_create_select_info(void *data, Evas_Object *parent)
{

	UG_TRACE_BEGIN;
	ugData *ugd = (ugData *)data;
	ug_mf_retvm_if(ugd == NULL, NULL, "ugd is NULL");
	ug_mf_retvm_if(parent == NULL, NULL, "parent is NULL");

	Evas_Object *pSelectInfo = NULL;
	pSelectInfo = elm_notify_add(parent);
	elm_notify_orient_set(pSelectInfo, ELM_NOTIFY_ORIENT_BOTTOM);

	evas_object_smart_callback_add(pSelectInfo, "selectioninfo,timeout", mf_ug_cb_select_info_timeout_cb, ugd);
	evas_object_event_callback_add(pSelectInfo, EVAS_CALLBACK_SHOW, mf_ug_cb_select_info_show_cb, ugd);
	evas_object_event_callback_add(pSelectInfo, EVAS_CALLBACK_HIDE, mf_ug_cb_select_info_hide_cb, ugd);
	UG_TRACE_END;

	return pSelectInfo;
}

static Evas_Object *__mf_ug_navi_bar_create_select_info_layout(Evas_Object *parent, Evas_Object *select_info)
{
	UG_TRACE_BEGIN;
	ug_mf_retvm_if(parent == NULL, NULL, "parent is NULL");
	ug_mf_retvm_if(select_info == NULL, NULL, "select_info is NULL");

	Evas_Object *pSelectInfoLayout = NULL;

	pSelectInfoLayout = elm_layout_add(parent);
	//elm_layout_theme_set(pSelectInfoLayout, "standard", "selectioninfo", "bottom_86");
	elm_object_content_set(select_info, pSelectInfoLayout);

	UG_TRACE_END;
	return pSelectInfoLayout;
}

void mf_ug_navi_bar_set_select_info_label(void *data, int dir_count, int file_count)
{
	UG_TRACE_BEGIN;
	ugData *ugd = (ugData *)data;
	ug_mf_retm_if(ugd == NULL, "ugd is NULL");

	ug_debug("dir count is [%d], file count is [%d]", dir_count, file_count);

	if (ugd->ug_MainWindow.ug_pLabel) {
		if (ugd->ug_MainWindow.ug_pSelectInfoLayout) {
			evas_object_del(ugd->ug_MainWindow.ug_pSelectInfoLayout);
			ugd->ug_MainWindow.ug_pSelectInfoLayout = NULL;
		}
		if (ugd->ug_MainWindow.ug_pLabel) {
			evas_object_del(ugd->ug_MainWindow.ug_pLabel);
			ugd->ug_MainWindow.ug_pLabel = NULL;
		}
	}

	if (!ugd->ug_MainWindow.ug_pLabel) {
		/*Note: selectioninfo is kind of popup, so, do not swallow it to part of layout
		   //And it should be attached on main layout because it calculate the position of itself by layout size (elm_layout_theme_set())
		   //Evas_Object *countlabel = elm_selectioninfo_add(ugd->ug_MainWindow.ug_pMainLayout);
		 */

		ugd->ug_MainWindow.ug_pLabel = __mf_ug_navi_bar_create_select_info(ugd, ugd->ug_MainWindow.ug_pMainLayout);

		ugd->ug_MainWindow.ug_pSelectInfoLayout =
			__mf_ug_navi_bar_create_select_info_layout(ugd->ug_MainWindow.ug_pMainLayout, ugd->ug_MainWindow.ug_pLabel);
		elm_layout_theme_set(ugd->ug_MainWindow.ug_pSelectInfoLayout, "standard", "selectioninfo", "vertical/bottom_64");

	}

	if (dir_count + file_count > 0) {
		char lable_dir[UG_LABEL_STRING_LENGTH] = { '\0', };
		char lable_file[UG_LABEL_STRING_LENGTH] = { '\0', };
		char *select_lable = NULL;
		/*1 TODO: need to update for multi-language */

		snprintf(lable_file, sizeof(lable_file), "%d", (dir_count+file_count));
		select_lable = g_strconcat("Selected (", lable_file, ")", NULL);

		elm_object_part_text_set(ugd->ug_MainWindow.ug_pSelectInfoLayout, "elm.text", select_lable);
		edje_object_part_text_set(_EDJ(ugd->ug_MainWindow.ug_pSelectInfoLayout), "elm.text", select_lable);
		elm_notify_timeout_set(ugd->ug_MainWindow.ug_pLabel, 3);
		evas_object_show(ugd->ug_MainWindow.ug_pLabel);
	} else {
		evas_object_hide(ugd->ug_MainWindow.ug_pLabel);
	}
	UG_TRACE_END;

	return;
}

/******************************
** Prototype    : mf_ug_navi_bar_create_navi_bar
** Description  :
** Input        : Evas_Object* parent
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
Evas_Object *mf_ug_navi_bar_create_navi_bar(Evas_Object *parent)
{
	UG_TRACE_BEGIN;
	Evas_Object *navi_bar = NULL;
	ug_mf_retvm_if(parent == NULL, NULL, "parent is NULL");

	navi_bar = elm_naviframe_add(parent);
	ug_mf_retvm_if(navi_bar == NULL, NULL, "Failed elm_navigationbar_add");
	elm_naviframe_prev_btn_auto_pushed_set(navi_bar, EINA_FALSE);

	evas_object_show(navi_bar);

	UG_TRACE_END;
	return navi_bar;
}


/******************************
** Prototype    : mf_ug_navi_bar_set_new_content
** Description  : Samsung
** Input        : Evas_Object *pLayout
**                Evas_Object *NaviContent
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_ug_navi_bar_set_new_content(Evas_Object *pLayout, Evas_Object *NaviContent)
{
	UG_TRACE_BEGIN;
	Evas_Object *unUsed = elm_object_part_content_unset(pLayout, "part1");
	evas_object_del(unUsed);
	elm_object_part_content_set(pLayout, "part1", NaviContent);
	UG_TRACE_END;
}

Evas_Object *mf_ug_navi_bar_create_box(Evas_Object * parent)
{
	UG_TRACE_BEGIN;
	ug_mf_retvm_if(parent == NULL, NULL, "parent is NULL");

	Evas_Object *box = NULL;
	box = elm_box_add(parent);
	ug_mf_retvm_if(box == NULL, NULL, "box is NULL");
	elm_object_focus_set(box, EINA_FALSE);
	evas_object_size_hint_weight_set(box, EVAS_HINT_EXPAND, EVAS_HINT_FILL);
	evas_object_size_hint_align_set(box, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_box_clear(box);
	evas_object_show(box);
	UG_TRACE_END;
	return box;

}

Evas_Object *mf_ug_navi_bar_create_layout(Evas_Object *parent, const char *edj, const char *grp_name)
{
	UG_TRACE_BEGIN;
	ug_mf_retvm_if(parent == NULL, NULL, "parent is NULL");
	ug_mf_retvm_if(edj == NULL, NULL, "edj is NULL");
	ug_mf_retvm_if(grp_name == NULL, NULL, "grp_name is NULL");

	Evas_Object *layout = NULL;

	layout = elm_layout_add(parent);
	ug_mf_retvm_if(layout == NULL, NULL, "layout is NULL");
	elm_object_focus_set(layout, EINA_FALSE);
	elm_layout_file_set(layout, edj, grp_name);
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(layout, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_show(layout);
	UG_TRACE_END;
	return layout;

}

/******************************
** Prototype    : mf_ug_navi_bar_create_default_view
** Description  : Samsung
** Input        : void *data
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
Evas_Object *__mf_ug_navi_bar_pathinfo_create(Evas_Object *parent, const char *content)
{
	UG_TRACE_BEGIN;
	ug_mf_retvm_if(parent == NULL, NULL, "parent is NULL");
	ug_mf_retvm_if(content == NULL, NULL, "content is NULL");

	Evas_Object *layout = NULL;

	layout = elm_layout_add(parent);
	elm_object_focus_set(layout, EINA_FALSE);
	elm_layout_file_set(layout, UG_EDJ_NAVIGATIONBAR, UG_GRP_PATH_INFO);
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(layout, EVAS_HINT_FILL, EVAS_HINT_FILL);

	elm_object_part_text_set(layout, "info", content);
	evas_object_show(layout);
	UG_TRACE_END;
	return layout;
}

void mf_ug_navi_bar_title_set(void *data)
{
	UG_TRACE_BEGIN;
	ugData *ugd = (ugData *)data;
	ug_mf_retm_if(ugd == NULL, "ugd is NULL");

	if (ugd->ug_UiGadget.ug_iSelectMode == IMPORT_MODE) {
		ug_debug();
		elm_object_part_text_set(ugd->ug_MainWindow.ug_pNaviBar, "elm.text.title", MF_UG_LABEL_IMPORT);
		elm_naviframe_item_title_visible_set(ugd->ug_MainWindow.ug_pNaviItem, EINA_TRUE);
		mf_ug_search_view_enter_search_routine(ugd, NULL, NULL);
	} else if ((ugd->ug_Status.ug_bSettingEntry == true && ugd->ug_Status.ug_bInSettingView == true)) {
		ug_debug();
		elm_naviframe_item_title_visible_set(ugd->ug_MainWindow.ug_pNaviItem, EINA_TRUE);
	} else if (!mf_ug_fm_svc_wapper_is_root_path(ugd)) {
		ug_debug();
		Evas_Object *home_btn = elm_button_add(ugd->ug_MainWindow.ug_pNaviBar);
		elm_object_style_set(home_btn, "naviframe/title/default");
		elm_object_text_set(home_btn, MF_UG_LABEL_HOME);
		evas_object_smart_callback_add(home_btn, "clicked", mf_ug_cb_home_button_cb, ugd);
		evas_object_show(home_btn);
		elm_object_item_part_content_set(ugd->ug_MainWindow.ug_pNaviItem, "title_left_btn", home_btn);

		Evas_Object *up_btn = elm_button_add(ugd->ug_MainWindow.ug_pNaviBar);
		elm_object_style_set(up_btn, "naviframe/title/default");
		elm_object_text_set(up_btn, MF_UG_LABEL_UPPER);
		evas_object_smart_callback_add(up_btn, "clicked", mf_ug_cb_upper_click_cb, ugd);
		evas_object_show(up_btn);
		elm_object_item_part_content_set(ugd->ug_MainWindow.ug_pNaviItem, "title_right_btn", up_btn);
		elm_naviframe_item_title_visible_set(ugd->ug_MainWindow.ug_pNaviItem, EINA_TRUE);
	} else if (ugd->ug_Status.ug_iMmcFlag == MMC_OFF && mf_ug_fm_svc_wapper_is_root_path(ugd)) {
		ug_debug();
		elm_naviframe_item_title_visible_set(ugd->ug_MainWindow.ug_pNaviItem, EINA_TRUE);
	} else {
		ug_debug();
		elm_naviframe_item_title_visible_set(ugd->ug_MainWindow.ug_pNaviItem, EINA_FALSE);
	}
}


void mf_ug_navi_bar_create_default_view(void *data)
{
	UG_TRACE_BEGIN;
	ugData *ugd = (ugData *)data;
	ug_mf_retm_if(ugd == NULL, "ugd is NULL");

	GString *title = NULL;
	Evas_Object *pNaviLayout = NULL;
	Evas_Object *newContent = NULL;
	Evas_Object *box = NULL;
	Evas_Object *top_layout = NULL;
	Evas_Object *upper_genlist = NULL;
	ugNaviBar *pNavi_s = mf_ug_navi_bar_get_in_use_navi(ugd);
	ug_mf_retm_if(pNavi_s == NULL, "get navigation bar in use failed");


	/*if the muisc is playing, destory the play */
	if (0 != ugd->ug_ListPlay.ug_Player) {
		mf_ug_list_play_destory_playing_file(ugd);
	}

	box = mf_ug_navi_bar_create_box(ugd->ug_MainWindow.ug_pNaviBar);
	pNaviLayout = mf_ug_navi_bar_create_layout(box, UG_EDJ_NAVIGATIONBAR, UG_GRP_NAVI_VIEW);
	ugd->ug_MainWindow.ug_pNaviLayout = pNaviLayout;

	if (ugd->ug_UiGadget.ug_iSelectMode == IMPORT_MODE) {
		newContent = __mf_ug_genlist_create_gl(ugd);
		ugd->ug_MainWindow.ug_pContent = newContent;

	} else {
		if (ugd->ug_Status.ug_bSettingEntry == true && ugd->ug_Status.ug_bInSettingView == true) {
			top_layout = mf_ug_navi_bar_create_box(box);
			evas_object_size_hint_min_set(top_layout, MF_UG_UPPER_WIDTH, MF_UG_UPPER_HEIGHT);
			upper_genlist = mf_navi_bar_upper_create(top_layout);
			mf_ug_genlist_goto_myfile_style_set(ugd);
			mf_ug_genlist_goto_myfile_item_append(upper_genlist, ugd);
			ugd->ug_Status.ug_bUpperFlag = true;
			elm_box_pack_end(top_layout, upper_genlist);
		} else if (!mf_ug_fm_svc_wapper_is_root_path(ugd)){
			Evas_Object *pathinfo_layout = NULL;
			Evas_Object *upper_layout = NULL;
			char *pathinfo = NULL;
			char *info = NULL;

			top_layout = mf_ug_navi_bar_create_box(box);
			evas_object_size_hint_weight_set(top_layout, EVAS_HINT_EXPAND, EVAS_HINT_FILL);
			evas_object_size_hint_align_set(top_layout, EVAS_HINT_FILL, EVAS_HINT_FILL);
			elm_box_clear(top_layout);
			//evas_object_size_hint_min_set(top_layout, MF_UG_UPPER_WIDTH, MF_UG_UPPER_HEIGHT);

			pathinfo = mf_ug_fm_svc_wapper_path_info_get(ugd->ug_Status.ug_pPath->str);
			info = mf_ug_fm_svc_path_info_translate(pathinfo, MF_UG_PATH_INFO_MAX_LENGTH_PORTRAIT);

			//Evas_Object *genlist = mf_ug_genlist_create_path_info(top_layout, &ugd->ug_Status.ug_pathinfoitec, info);

			pathinfo_layout = __mf_ug_navi_bar_pathinfo_create(top_layout, info);

			elm_box_pack_end(top_layout, pathinfo_layout);
			ugd->ug_Status.ug_bUpperFlag = true;
		}

		if (top_layout) {
			elm_box_pack_start(box, top_layout);
		} else {
			ugd->ug_Status.ug_bUpperFlag = true;
		}
		/*set content */
		if (ugd->ug_UiGadget.ug_iSelectMode == SINGLE_ALL_MODE || ugd->ug_UiGadget.ug_iSelectMode == SINGLE_FILE_MODE) {
			__mf_ug_navi_bar_create_group_radio_box(ugd);
		}
		newContent = mf_ug_genlist_create_content_list_view(ugd);
		ugd->ug_MainWindow.ug_pContent = newContent;
		/*set title segment or title */
		if (mf_ug_fm_svc_wapper_is_root_path(ugd)) {
			title = g_string_new(MF_UG_LABEL_MYFILES);
		} else {
			title = mf_ug_fm_svc_wapper_get_file_name(ugd->ug_Status.ug_pPath);
		}

		if (title != NULL) {
			pNavi_s->ug_pNaviTitle = g_strdup(title->str);
			g_string_free(title, TRUE);
			title = NULL;
		}
	}

	/*navigation view integration */
	evas_object_show(newContent);
	elm_object_part_content_set(pNaviLayout, "part1", newContent);
	elm_box_pack_end(box, pNaviLayout);
	ugd->ug_MainWindow.ug_pNaviBox = box;
	__mf_ug_navi_bar_push_content(ugd, box);

	/*add control bar for navigation bar */
	mf_ug_ctrl_bar_set(ugd, pNavi_s);

	mf_ug_navi_bar_title_set(ugd);
	mf_ug_navi_bar_remove_previous_contents(ugd);
	UG_TRACE_END;
}


void mf_ug_navi_bar_create_new_folder_view(void *data)
{

	UG_TRACE_BEGIN;
	ugData *ugd = (ugData *)data;
	ug_mf_retm_if(ugd == NULL, "ugd is NULL");

	Evas_Object *newContent = NULL;

	Evas_Object *pNaviBar = NULL;
	GString *title = NULL;

	mf_ug_navi_bar_set_navi_to_main(ugd);

	ugNaviBar *pNavi_s = mf_ug_navi_bar_get_in_use_navi(ugd);
	ug_mf_retm_if(pNavi_s == NULL, "pNavi_s is NULL");

	pNaviBar = ugd->ug_MainWindow.ug_pNaviBar;

	/*set content*/
	newContent = mf_ug_genlist_create_list_new_folder_style(ugd);
	/*set title segment or title*/
	title = g_string_new(MF_UG_LABEL_CREATE);

	if (title != NULL) {
		pNavi_s->ug_pNaviTitle = g_strdup(title->str);
		g_string_free(title, TRUE);
		title = NULL;
	}
	/*navigation view integration*/
	__mf_ug_navi_bar_push_content(ugd, newContent);

	Evas_Object *l_bt = NULL;
	Evas_Object *r_bt = NULL;
	r_bt = elm_button_add(pNaviBar);
	elm_object_focus_set(r_bt, EINA_FALSE);
	elm_object_style_set(r_bt, "naviframe/title/default");
	elm_object_text_set(r_bt, MF_UG_LABEL_CANCEL);
	evas_object_smart_callback_add(r_bt, "clicked", mf_ug_cb_cancel_new_folder_cb, ugd);
	elm_object_item_part_content_set(ugd->ug_MainWindow.ug_pNaviItem, "title_right_btn", r_bt);

	l_bt = elm_button_add(pNaviBar);
	elm_object_focus_set(l_bt, EINA_FALSE);
	elm_object_style_set(l_bt, "naviframe/title/default");
	elm_object_text_set(l_bt, MF_UG_LABEL_SAVE);
	evas_object_smart_callback_add(l_bt, "clicked", mf_ug_cb_save_cb, ugd);
	elm_object_item_part_content_set(ugd->ug_MainWindow.ug_pNaviItem, "title_left_btn", l_bt);



}

/******************************
** Prototype    : mf_ug_navi_bar_get_in_use_navi
** Description  : Samsung
** Input        : void* data
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
ugNaviBar *mf_ug_navi_bar_get_in_use_navi(void *data)
{
	UG_TRACE_BEGIN;
	ugData *ugd = (ugData *)data;
	ug_mf_retvm_if(ugd == NULL, NULL, "ugd is NULL");

	Eina_List *pNaviList = ugd->ug_MainWindow.ug_pNaviBarList;
	ug_mf_retvm_if(pNaviList == NULL, NULL, "pNavi_sList is NULL");

	ugNaviBar *ret = NULL;
	Eina_List *l = NULL;
	void *item;

	EINA_LIST_FOREACH(pNaviList, l, item) {
		ret = (ugNaviBar *)item;
		if (ret->ug_bNaviFlagInUse == TRUE) {
			break;
		} else {
			ret = NULL;
		}
	}
	UG_TRACE_END;
	return ret;
}


/******************************
** Prototype    : mf_ug_navi_bar_reset_navi_object
** Description  : Samsung
** Input        : ugNaviBar *pNavi_s
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_ug_navi_bar_reset_navi_object(ugNaviBar *pNavi_s)
{
	UG_TRACE_BEGIN;
	ug_mf_retm_if(pNavi_s == NULL, "pNavi_s is NULL");

	pNavi_s->ug_pNaviTitle = NULL;
	if (pNavi_s->ug_pCurrentPath != NULL) {
		free(pNavi_s->ug_pCurrentPath);
		pNavi_s->ug_pCurrentPath = NULL;
	}
	if (pNavi_s->ug_pNaviLabel != NULL)
		free(pNavi_s->ug_pNaviLabel);
	pNavi_s->ug_pNaviLabel = NULL;
	UG_TRACE_END;
}

/******************************
** Prototype    : mf_ug_navi_bar_get_navi_struct_by_label
** Description  : Samsung
** Input        : void* data
**                const char* pNaviLabel
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
ugNaviBar *mf_ug_navi_bar_get_navi_struct_by_label(void *data, const char *pNaviLabel)
{
	UG_TRACE_BEGIN;
	ugData *ugd = (ugData *)data;
	ug_mf_retvm_if(ugd == NULL, NULL, "ugd is NULL");
	ug_mf_retvm_if(pNaviLabel == NULL, NULL, "pNaviLabel is NULL");

	Eina_List *l = NULL;
	void *item = NULL;

	ugNaviBar *ret = NULL;
	EINA_LIST_FOREACH(ugd->ug_MainWindow.ug_pNaviBarList, l, item) {
		if (g_strcmp0(((ugNaviBar *)item)->ug_pNaviLabel, pNaviLabel) == 0) {
			ret = (ugNaviBar *)item;
			break;
		}
	}
	UG_TRACE_END;
	return ret;
}


/******************************
** Prototype    : mf_ug_navi_bar_remove_navi_list_item_by_label
** Description  : Samsung
** Input        : void *data
**                const char* pNaviLabel
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_ug_navi_bar_remove_navi_list_item_by_label(void *data, const char *pNaviLabel)
{
	UG_TRACE_BEGIN;
	ugData *ugd = (ugData *)data;
	ug_mf_retm_if(ugd == NULL, "ugd is NULL");
	ug_mf_retm_if(pNaviLabel == NULL, "pNaviLabel is NULL");

	bool bInUseFlag = FALSE;

	ugNaviBar *pNavi_s = mf_ug_navi_bar_get_navi_struct_by_label(ugd, pNaviLabel);

	if (pNavi_s == NULL) {
		return;
	}
	Eina_List *plistNaviMMC = eina_list_data_find_list(ugd->ug_MainWindow.ug_pNaviBarList, pNavi_s);
	ugd->ug_MainWindow.ug_pNaviBarList = eina_list_remove_list(ugd->ug_MainWindow.ug_pNaviBarList, plistNaviMMC);

	if (pNavi_s->ug_bNaviFlagInUse == TRUE) {
		bInUseFlag = TRUE;
	}

	mf_ug_navi_bar_reset_navi_object(pNavi_s);

	if (bInUseFlag == TRUE) {
		pNavi_s = (ugNaviBar *)eina_list_nth(ugd->ug_MainWindow.ug_pNaviBarList, 0);
		pNavi_s->ug_bNaviFlagInUse = TRUE;
	}
	UG_TRACE_END;
}


/******************************
** Prototype    : mf_ug_navi_bar_refresh_navibar_recovered_view
** Description  : Samsung
** Input        : void *data
**                ugNaviBar* pNaviStruct
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_ug_navi_bar_refresh_navibar_recovered_view(void *data, ugNaviBar *pNaviStruct)
{
	UG_TRACE_BEGIN;
	ugData *ugd = (ugData *)data;
	ug_mf_retm_if(ugd == NULL, "ugd is NULL");
	ug_mf_retm_if(pNaviStruct == NULL, "pNaviStruct is NULL");

	if (ugd->ug_Status.ug_pPath != NULL) {
		g_string_free(ugd->ug_Status.ug_pPath, TRUE);
		ugd->ug_Status.ug_pPath = NULL;
	}
	ugd->ug_Status.ug_pPath = g_string_new(pNaviStruct->ug_pCurrentPath);
	Evas_Object *newContent = mf_ug_genlist_create_content_list_view(ugd);

	ugd->ug_MainWindow.ug_pContent = newContent;
	mf_ug_navi_bar_set_new_content(ugd->ug_MainWindow.ug_pNaviLayout, newContent);
	UG_TRACE_END;
}

void mf_ug_navi_bar_transition_finished_cb(void *data, Evas_Object *obj, void *event_info)
{
 	UG_TRACE_BEGIN;
	ug_mf_retm_if(data == NULL, "data is NULL");
	ug_mf_retm_if(obj == NULL, "obj is NULL");
	ugData *ugd = (ugData *)data;
	Elm_Object_Item *it = ugd->ug_MainWindow.ug_pPreNaviItem;
	UG_SAFE_DEL_NAVI_ITEM(it);
	ugd->ug_MainWindow.ug_pPreNaviItem = NULL;
	mf_ug_navi_bar_set_blocking(data, false);
        evas_object_smart_callback_del(obj, "transition,finished", mf_ug_navi_bar_transition_finished_cb);
 	UG_TRACE_END;
}

void mf_ug_navi_bar_remove_previous_contents(void *data)
{
 	UG_TRACE_BEGIN;

	ug_mf_retm_if(data == NULL, "data is NULL");
	ugData *ugd = (ugData *)data;
	Evas_Object *pNaviBar = NULL;
	if (ugd->ug_MainWindow.ug_pNaviBar) {
		pNaviBar = ugd->ug_MainWindow.ug_pNaviBar;
		mf_ug_navi_bar_set_blocking(data, true);
		evas_object_smart_callback_add(pNaviBar, "transition,finished", mf_ug_navi_bar_transition_finished_cb, ugd);
	}
 	UG_TRACE_END;
}

void mf_ug_navi_bar_del_item_directly(void *data)
{
	UG_TRACE_BEGIN;
	ug_mf_retm_if(data == NULL, "data is NULL");
	ugData *ugd = (ugData *)data;

	if (ugd->ug_Status.ug_b_NaviPushFlag) {
		ugNaviBar *pNavi_s_previous = NULL;
		pNavi_s_previous = mf_ug_navi_bar_get_in_use_navi(ugd) ;
		if(pNavi_s_previous && ugd->ug_MainWindow.ug_pPreNaviItem)
		{
			UG_SAFE_DEL_NAVI_ITEM(ugd->ug_MainWindow.ug_pPreNaviItem);
			mf_ug_navi_bar_set_blocking(ugd, false);
			evas_object_smart_callback_del(ugd->ug_MainWindow.ug_pNaviBar, "transition,finished", mf_ug_navi_bar_transition_finished_cb);
	       }
	}
 	UG_TRACE_END;
}


void mf_ug_navi_bar_set_blocking(void *data, bool bblock_during_transition)
{
	ug_mf_retm_if(data == NULL, "data is NULL");
	ugData *ugd = (ugData *)data;

	ugd->ug_Status.ug_bCancelDisableFlag = bblock_during_transition;
	ugd->ug_Status.ug_b_NaviPushFlag = bblock_during_transition;
	return;
}

ugNaviBar *ug_mf_navi_bar_get_navi_from_navilist(Eina_List *navilist, const char *label)
{

	UG_TRACE_BEGIN;
	ug_mf_retvm_if(navilist == NULL, NULL, "navilist is NULL");
	ug_mf_retvm_if(label == NULL, NULL, "label is NULL");

	void *data = NULL;
	ugNaviBar *pNavi = NULL;
	Eina_List *l = NULL;
	EINA_LIST_FOREACH(navilist, l, data) {
		if (data) {
			pNavi = (ugNaviBar *)data;
			if (pNavi && pNavi->ug_pNaviLabel && g_strcmp0(label, pNavi->ug_pNaviLabel) == 0)
				return pNavi;
			else
				continue;
		}
	}
	return NULL;
}

void mf_ug_navi_bar_set_navi_to_main(void *data)
{
	UG_TRACE_BEGIN;
	ug_mf_retm_if(data == NULL, "data is NULL");
	ugData *ugd = (ugData *)data;

	if (ugd->ug_Status.ug_iMmcFlag != MMC_OFF) {
		elm_object_part_content_unset(ugd->ug_MainWindow.ug_pTabLayout, "elm.swallow.content");
		evas_object_hide(ugd->ug_MainWindow.ug_pTabBar);
		elm_object_part_content_unset(ugd->ug_MainWindow.ug_pTabLayout, "elm.swallow.tabbar");
		evas_object_hide(ugd->ug_MainWindow.ug_pTabLayout);
		elm_object_part_content_unset(ugd->ug_MainWindow.ug_pMainLayout, "elm.swallow.content");
		elm_object_part_content_set(ugd->ug_MainWindow.ug_pMainLayout, "elm.swallow.content", ugd->ug_MainWindow.ug_pNaviBar);
		evas_object_show(ugd->ug_MainWindow.ug_pMainLayout);
	}
	UG_TRACE_END;
}

void mf_ug_navi_bar_reset_tab_to_main(void *data)
{
	UG_TRACE_BEGIN;
	ug_mf_retm_if(data == NULL, "data is NULL");
	ugData *ugd = (ugData *)data;
	ugNaviBar *pNavi_s = mf_ug_navi_bar_get_in_use_navi(ugd);

	if (ugd->ug_Status.ug_iMmcFlag != MMC_OFF) {
		if (mf_ug_fm_svc_wapper_is_root_path(ugd)) {
			Evas_Object *pOldContent = NULL;
			pOldContent = elm_object_part_content_unset(ugd->ug_MainWindow.ug_pMainLayout, "elm.swallow.content");
			elm_object_part_content_set(ugd->ug_MainWindow.ug_pTabLayout, "elm.swallow.content", pOldContent);
			elm_object_part_content_set(ugd->ug_MainWindow.ug_pTabLayout, "elm.swallow.tabbar", ugd->ug_MainWindow.ug_pTabBar);
			elm_object_part_content_set(ugd->ug_MainWindow.ug_pMainLayout, "elm.swallow.content", ugd->ug_MainWindow.ug_pTabLayout);
			evas_object_show(ugd->ug_MainWindow.ug_pTabLayout);
			evas_object_show(ugd->ug_MainWindow.ug_pTabBar);
			mf_ug_tab_bar_set_selected_item(mf_ug_tab_bar_get_item_by_label(ugd->ug_MainWindow.ug_pTabBar, pNavi_s->ug_pNaviLabel));
		}
	} else {
		elm_object_part_content_unset(ugd->ug_MainWindow.ug_pMainLayout, "elm.swallow.content");
		elm_object_part_content_set(ugd->ug_MainWindow.ug_pMainLayout, "elm.swallow.content", ugd->ug_MainWindow.ug_pNaviBar);
		elm_naviframe_item_title_visible_set(ugd->ug_MainWindow.ug_pNaviItem, EINA_TRUE);
	}
	UG_TRACE_END;
}

