/*
 * myfile
 *
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.tizenopensource.org/license
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */



#include "mf-ug-winset.h"
#include "mf-ug-util.h"
#include "mf-ug-fs-util.h"
#include "mf-ug-fm-svc-wrapper.h"
#include "mf-ug-cb.h"
#include "mf-ug-resource.h"

/******************************
** Prototype    : mf_ug_tab_bar_create
** Description  : Samsung
** Input        : Evas_Object *parent
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
Evas_Object *mf_ug_tab_bar_create(Evas_Object *parent)
{
	UG_TRACE_BEGIN;
	ug_mf_retv_if(parent == NULL, NULL);
	Evas_Object *toolbar = elm_toolbar_add(parent);
	elm_toolbar_shrink_mode_set(toolbar, ELM_TOOLBAR_SHRINK_EXPAND);
	elm_object_style_set(toolbar, "tabbar");
	elm_toolbar_select_mode_set(toolbar, ELM_OBJECT_SELECT_MODE_ALWAYS);
	elm_toolbar_homogeneous_set(toolbar, EINA_FALSE);

	UG_TRACE_BEGIN;
	return toolbar;
}


/******************************
** Prototype    : mf_ug_tab_bar_append_item
** Description  : Samsung
** Input        : Evas_Object *pTabBar
**                char *pTabIconPath
**                char *pTabBarLabel
**                Evas_Object *pTabView
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
Elm_Object_Item *mf_ug_tab_bar_append_item(Evas_Object *pTabBar, char *pTabIconPath, char *pTabBarLabel, void *user_data)
{
	UG_TRACE_BEGIN;
	ug_mf_retv_if(pTabBar == NULL, NULL);

	Elm_Object_Item *pItem = NULL;
	pItem = elm_toolbar_item_append(pTabBar, pTabIconPath, pTabBarLabel, mf_ug_cb_tab_bar_cb, user_data);
	UG_TRACE_END;
	return pItem;
}


/******************************
** Prototype    : mf_ug_tab_bar_set_item_view
** Description  : Samsung
** Input        : Elm_Object_Item *it
**                Evas_Object * view
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_ug_tab_bar_set_item_view(Evas_Object *tab_layout, Evas_Object *view)
{
	UG_TRACE_BEGIN;
	ug_mf_retm_if(tab_layout == NULL, "tab_layout is NULL");
	ug_mf_retm_if(view == NULL, "view is NULL");
	Evas_Object *layout = tab_layout;
	Evas_Object *unset = elm_object_part_content_get(layout, "elm.swallow.content");
	if (unset) {
		elm_object_part_content_unset(layout, "elm.swallow.content");
		evas_object_hide(unset);
	}

	elm_object_part_content_set(layout, "elm.swallow.content", view);
	UG_TRACE_END;
}


/******************************
** Prototype    : mf_ug_tab_bar_remove_item_by_label
** Description  : Samsung
** Input        : Evas_Object *pTabBar
**                char *pTabLabel
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_ug_tab_bar_remove_item_by_label(Evas_Object *pTabBar, char *pTabLabel)
{
	UG_TRACE_BEGIN;
	Elm_Object_Item *pItem = NULL;
	pItem = elm_toolbar_first_item_get(pTabBar);

	while(pItem) {
		if (g_strcmp0(pTabLabel, elm_object_item_text_get(pItem)) == 0) {
			elm_object_item_del(pItem);
			return;
		}
		pItem = elm_toolbar_item_next_get(pItem);
	}
	UG_TRACE_END;
}

/******************************
** Prototype    : mf_ug_tab_bar_remove_all_items
** Description  : Samsung
** Input        : Evas_Object *pTabBar
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_ug_tab_bar_remove_all_items(Evas_Object *pTabBar)
{

	UG_TRACE_BEGIN;
	Elm_Object_Item *it = NULL;
	Elm_Object_Item *pNext = NULL;

	it = elm_toolbar_first_item_get(pTabBar);
	while(it) {
		pNext = elm_toolbar_item_next_get(it);
		elm_object_item_del(it);
		it = pNext;
	}
	UG_TRACE_END;
}


/******************************
** Prototype    : mf_ug_tab_bar_set_selected_item
** Description  : Samsung
** Input        : Elm_Object_Item* pItem
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_ug_tab_bar_set_selected_item(Elm_Object_Item *pItem)
{
	UG_TRACE_BEGIN;
	elm_toolbar_item_selected_set(pItem, EINA_TRUE);
	UG_TRACE_END;
}


/******************************
** Prototype    : mf_ug_tab_bar_set_item_disable
** Description  : Samsung
** Input        : Evas_Object *pTabBar
**                Eina_Bool disable
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void mf_ug_tab_bar_set_item_disable(Evas_Object *pTabBar, Eina_Bool disable)
{
	Elm_Object_Item *pCurrentItem = NULL;

	if (pTabBar != NULL) {

		pCurrentItem = elm_toolbar_first_item_get(pTabBar);
		while(pCurrentItem) {
			elm_object_item_disabled_set(pCurrentItem, disable);
			pCurrentItem = elm_toolbar_item_next_get(pCurrentItem);
		}
	}
}

int mf_ug_tab_bar_items_count_get(Evas_Object *pTabBar)
{
	ug_mf_retvm_if(pTabBar == NULL, 0, "pTabBar is NULL");

	Elm_Object_Item *pCurrentItem = NULL;
	int count = 0;

	pCurrentItem = elm_toolbar_first_item_get(pTabBar);
	while(pCurrentItem) {
		count++;
		pCurrentItem = elm_toolbar_item_next_get(pCurrentItem);
	}

	return count;
}

void  mf_ug_tar_bar_disable(void *data, Eina_Bool disabled)
{

	UG_TRACE_BEGIN;
	ugData *ugd = (ugData *)data;
	ug_mf_retm_if(ugd == NULL, "ugd is NULL");

	if (mf_ug_fm_svc_wapper_is_root_path(ugd)) {
		mf_ug_tab_bar_init(ugd);
	    if (ugd->ug_Status.ug_iMmcFlag != MMC_OFF && ugd->ug_MainWindow.ug_pTabBar) {
		    if (disabled) {
			    mf_ug_navi_bar_set_navi_to_main(ugd);
		    } else {
			    mf_ug_navi_bar_reset_tab_to_main(ugd);
		    }

	    }
	}
	UG_TRACE_END;
}

Elm_Object_Item *mf_ug_tab_bar_get_item_by_label(Evas_Object *pTabBar, char *pTabLabel)
{
	ug_mf_retvm_if(pTabBar == NULL, NULL, "pTabBar is NULL");
	ug_mf_retvm_if(pTabLabel == NULL, NULL, "pTabLabel is NULL");
	Elm_Object_Item *pItem = NULL;

	pItem = elm_toolbar_first_item_get(pTabBar);
	while(pItem) {
		if (g_strcmp0(pTabLabel, elm_object_item_text_get(pItem)) == 0) {
			return pItem;
		}
		pItem = elm_toolbar_item_next_get(pItem);
	}
	return NULL;
}

void mf_ug_tab_bar_init(void *data)
{
	UG_TRACE_BEGIN;
	ug_mf_retm_if(data == NULL, "data is null");
	ugData *ugd = (ugData *)data;

	if (ugd->ug_MainWindow.ug_pTabLayout == NULL) {
		ugd->ug_MainWindow.ug_pTabLayout = mf_ug_main_tab_layout_create(ugd->ug_MainWindow.ug_pWindow);
		ugd->ug_MainWindow.ug_pTabBackGround = mf_ug_main_create_bg(ugd->ug_MainWindow.ug_pWindow);
		elm_object_part_content_set(ugd->ug_MainWindow.ug_pTabLayout, "elm.swallow.bg", ugd->ug_MainWindow.ug_pTabBackGround);
	}
	if (ugd->ug_MainWindow.ug_pTabBar == NULL) {
		ugd->ug_MainWindow.ug_pTabBar = mf_ug_tab_bar_create(ugd->ug_MainWindow.ug_pWindow);
	}
	ugNaviBar *pNaviBarStruct = NULL;

	pNaviBarStruct = ug_mf_navi_bar_get_navi_from_navilist(ugd->ug_MainWindow.ug_pNaviBarList, MF_UG_LABEL_PHONE);
	if(!mf_ug_tab_bar_get_item_by_label(ugd->ug_MainWindow.ug_pTabBar, pNaviBarStruct->ug_pNaviLabel))
		mf_ug_tab_bar_append_item(ugd->ug_MainWindow.ug_pTabBar, UG_ICON_PHONE, MF_UG_LABEL_PHONE,
					  ugd);

	pNaviBarStruct = ug_mf_navi_bar_get_navi_from_navilist(ugd->ug_MainWindow.ug_pNaviBarList, MF_UG_LABEL_MMC);
	if(!mf_ug_tab_bar_get_item_by_label(ugd->ug_MainWindow.ug_pTabBar, MF_UG_LABEL_MMC))
		mf_ug_tab_bar_append_item(ugd->ug_MainWindow.ug_pTabBar, UG_ICON_MEMORY, MF_UG_LABEL_MMC,
					  ugd);

}


